function [b,Mnlin,Mlin,rss] = mixnngiter(y,X,Mnlin,Mlin,fx,tol)
% y: n*1
% x: n*p
% id: 2*p logical, first row - indicators of nonlinear covariates,
%     second row - indicators of linear covariates

if nargin < 6
    tol = 1e-3;
end

% fx = @(aa,xx) 1./(1+exp(-xx*diag(aa)))-0.5;
hf = @(bb,xx,qq) [fx(bb(1:qq),xx(:,1:qq)),xx(:,qq+1:end)]*bb(qq+1:end)';
n = length(y);
ystd = (y-mean(y))/std(y);

ssu = norm(y); ssc = 0; 
step = 0;
while abs(ssc-ssu) > tol
    
     ssc = ssu;
     
    %% initial estimates
    while step == 0
        xnlin = X(:,Mnlin); xlin = X(:,Mlin);
        q = length(Mnlin); p = q+length(Mlin);
        
        if q ~= 0
            hfmin = @(bb) norm(hf(bb,[xnlin,xlin],q)-y);
            [b0,s0] = ga(hfmin,p+q,[],[],[],[],0.01*ones(1,p+q),7*ones(1,p+q));
            [b0tmp,s0tmp] = fmincon(hfmin,b0,[],[],[],[],0.01*ones(1,p+q),7*ones(1,p+q),[],...
                optimset('Algorithm','sqp','MaxFunEvals',1000,'MaxIter',1000));
            if s0 > s0tmp
                b0 = b0tmp;
            end
            bnlin = reshape(b0(1:2*q),q,2)';
            idnlin = (bnlin(1,:)>0.5 & abs(bnlin(2,:))>tol);
            blin = b0(2*q+1:end);
            idlin = (abs(blin)>tol);
            
            if sum(idnlin)+sum(idlin) == p
                step = 1;
            else
                Mnlin = Mnlin(idnlin); Mlin = Mlin(idlin);
            end
        else
            step = 1;
        end             
    end

    %% iterative updates
    if q ~= 0
        z = [fx(bnlin(1,:),xnlin),xlin];   
    else
        z = xlin;
    end
    zstd = (z-repmat(mean(z),n,1))*diag(1./std(z));
    betaM0 = nng_finds(zstd,ystd,{'Method','kcv','stderror'})';
    betaM1 = betaM0*std(y)./std(z);
    ssu = norm(z*betaM1'-y);    
    blin = betaM1(q+1:end); idlin = abs(blin)>tol;
    if q == 0 || all(betaM1(1:q)==0) 
        b = blin(idlin); Mnlin = []; Mlin = Mlin(idlin); rss = ssu;
        return;
    else    
        bnlin(2,:) = betaM1(1:q); idnlin = abs(bnlin(2,:))>tol;
    end
    
    if sum(idnlin)+sum(idlin) < p || isempty(intersect(Mnlin,Mlin))
        bnlin = bnlin(:,idnlin); blin = blin(idlin);        
        Mnlin = Mnlin(idnlin); Mlin = Mlin(idlin);
        xnlin = xnlin(:,idnlin); xlin = xlin(:,idlin);
        q = length(Mnlin); p = q+length(Mlin);
        
        hfmin = @(bb) norm(fx(bb,xnlin)*bnlin(2,:)'+xlin*blin'-y);
        [b1,s1] = ga(hfmin,q,[],[],[],[],0.01*ones(1,q),7*ones(1,q));
        [b1tmp,s1tmp] = fmincon(hfmin,b1,[],[],[],[],0.01*ones(1,q),7*ones(1,q),[],...
            optimset('Algorithm','sqp','MaxFunEvals',1000,'MaxIter',1000));
        if s1 >= s1tmp
            ssu = s1tmp;
            bnlin(1,:) = b1tmp;
        else
            ssu = s1;
            bnlin(1,:) = b1;
        end
    else
        regmod = regstats(ystd,zstd,'linear',{'tstat'});
        pval = regmod.tstat.pval(2:end)';
        [ind,ia,ib] = intersect(Mnlin,Mlin);
        [mpval,ii] = max([pval(ia);pval(q+ib)]);
        del = find(mpval==max(mpval));
        if max(mpval) > 0.05
            if ii(del) == 1
                Mnlin = setdiff(Mnlin,ind(del));
            else
                Mlin = setdiff(Mlin,ind(del));
            end        
        else
            Mlin = setdiff(Mlin,ind(del));
        end
        step = 0;
    end
    
end

%%
p = length(Mnlin)+length(Mlin);
hfmin = @(bb) norm(hf(bb,[xnlin,xlin],q)-y);
% [b,rss] = ga(hfmin,pu+q,[],[],[],[],0.01*ones(1,pu+q),7*ones(1,pu+q),...
%     [],gaoptimset('Generations',1000));
[b,rss] = fmincon(hfmin,[bnlin(1,:),bnlin(2,:),blin],...
    [],[],[],[],0.01*ones(1,p+q),7*ones(1,p+q),[],...
    optimset('Algorithm','sqp','MaxFunEvals',1000,'MaxIter',1000));
% if rss > rsstmp
%     rss = rsstmp;
%     b = btmp;
% end



